@@ -5,6 +5,7 @@
 # ------------------------------------------------------------------------------
 
 import base64
+import binascii
 import os
 import tempfile
 from io import BytesIO
@@ -13,6 +14,7 @@ from urllib.request import urlopen
 from PIL import ImageFile, ImageTk
 from PIL.Image import BICUBIC, EXTENSION, Exif, Image, init, open as img_open
 
+from .ImageExtensions import *
 from ..Exceptions import ArgumentError
 from ..Files import *
 from ..Json import *
@@ -26,90 +28,17 @@ __all__ = [
     'ImageExtensions',
     ]
 
-# t = ['BLP', 'BMP', 'BUFR', 'CUR', 'DCX', 'DDS', 'DIB', 'EPS', 'FITS', 'FLI', 'FTEX', 'GBR', 'GIF', 'GRIB', 'HDF5', 'ICNS', 'ICO', 'IM', 'IMT', 'IPTC', 'JPEG', 'JPEG2000', 'MCIDAS', 'MPEG',
-#      'MSP', 'PCD', 'PCX', 'PIXAR', 'PNG', 'PPM', 'PSD', 'SGI', 'SPIDER', 'SUN', 'TGA', 'TIFF', 'WEBP', 'WMF', 'XBM', 'XPM', 'XVTHUMB']
-
-class ImageExtensions(str, Enum):
-    blp = '.blp'
-    bmp = '.bmp'
-    dib = '.dib'
-    bufr = '.bufr'
-    cur = '.cur'
-    pcx = '.pcx'
-    dcx = '.dcx'
-    dds = '.dds'
-    ps = '.ps'
-    eps = '.eps'
-    fit = '.fit'
-    fits = '.fits'
-    fli = '.fli'
-    flc = '.flc'
-    ftc = '.ftc'
-    ftu = '.ftu'
-    gbr = '.gbr'
-    gif = '.gif'
-    grib = '.grib'
-    h5 = '.h5'
-    hdf = '.hdf'
-    png = '.png'
-    apng = '.apng'
-    jp2 = '.jp2'
-    j2k = '.j2k'
-    jpc = '.jpc'
-    jpf = '.jpf'
-    jpx = '.jpx'
-    j2c = '.j2c'
-    icns = '.icns'
-    ico = '.ico'
-    im = '.im'
-    iim = '.iim'
-    tif = '.tif'
-    tiff = '.tiff'
-    jfif = '.jfif'
-    jpe = '.jpe'
-    jpg = '.jpg'
-    jpeg = '.jpeg'
-    mpg = '.mpg'
-    mpeg = '.mpeg'
-    mpo = '.mpo'
-    msp = '.msp'
-    palm = '.palm'
-    pcd = '.pcd'
-    pdf = '.pdf'
-    pxr = '.pxr'
-    pbm = '.pbm'
-    pgm = '.pgm'
-    ppm = '.ppm'
-    pnm = '.pnm'
-    psd = '.psd'
-    bw = '.bw'
-    rgb = '.rgb'
-    rgba = '.rgba'
-    sgi = '.sgi'
-    ras = '.ras'
-    tga = '.tga'
-    icb = '.icb'
-    vda = '.vda'
-    vst = '.vst'
-    webp = '.webp'
-    wmf = '.wmf'
-    emf = '.emf'
-    xbm = '.xbm'
-    xpm = '.xpm'
-
-
-
 class ImageObject(object):
-    __slots__ = ['_img', '_path', '_fp', '_MaxWidth', '_MaxHeight', '_name_']
+    __slots__ = ['_img', '_path', '_fp', '_widthMax', '_heightMax', '_name_']
     _path: Path
     _fp: Optional[BytesIO]
     _img: Image
-    _MaxWidth: Optional[int]
-    _MaxHeight: Optional[int]
-    def __init__(self, img: Optional[Image], MaxWidth: Optional[int] = None, MaxHeight: Optional[int] = None, *, LOAD_TRUNCATEDImageS: bool = True):
+    _widthMax: Optional[int]
+    _heightMax: Optional[int]
+    def __init__(self, img: Optional[Image], widthMax: Optional[int] = None, heightMax: Optional[int] = None, *, LOAD_TRUNCATEDImageS: bool = True):
         self._img = img
-        self.SetMaxSize(MaxWidth, MaxHeight)
-        if MaxHeight and MaxWidth:
+        self.SetMaxSize(widthMax, heightMax)
+        if widthMax and heightMax:
             self.Resize(check_metadata=True)
         ImageFile.LOAD_TRUNCATEDImageS = LOAD_TRUNCATEDImageS
     def __hash__(self): return hash(self._img)
@@ -144,21 +73,21 @@ class ImageObject(object):
         if not path and not extension:
             raise ArgumentError('Must Provide either the file path or the image type extension')
 
-        self._path = path or self._TempFilePath(extension)
+        self._path = path or self._TempFilePath(extension)  # ImageExtensions(path.extension({ '.': '' })
         return self
     def save(self): self._img.save(self._fp)
 
-    def SetMaxSize(self, width: Optional[int], height: Optional[int]):
-        self._MaxWidth = height
-        self._MaxHeight = width
+    def SetMaxSize(self, widthMax: Optional[int], heightMax: Optional[int]):
+        self._widthMax = widthMax
+        self._heightMax = heightMax
 
     @property
     def size(self) -> Size: return Size.Create(self._img.width, self._img.height)
 
     @property
     def _factors(self) -> Tuple[float, float]:
-        if isinstance(self._MaxWidth, int) and isinstance(self._MaxHeight, int):
-            return self._MaxWidth / self._img.width, self._MaxHeight / self._img.height
+        if isinstance(self._widthMax, int) and isinstance(self._heightMax, int):
+            return self._widthMax / self._img.width, self._heightMax / self._img.height
 
         return 1, 1
     def Maximum_ScalingFactor(self) -> float: return max(self._factors)
@@ -171,7 +100,7 @@ class ImageObject(object):
 
     def Rotate(self, angle: int = None, *, expand: bool = True, Offset: Tuple[int, int] = None, fill_color=None, center=None, resample=BICUBIC) -> 'ImageObject':
         """
-            CAUTION: Offset will TRIM the image if moved out of bounds of the 
+            CAUTION: Offset will TRIM the image if moved out of bounds of the
 
         :param fill_color:
         :param center:
@@ -217,46 +146,81 @@ class ImageObject(object):
 
         self._img = self._img.resize(**kwargs)
         return self
-    def ToPhotoImage(self) -> ImageTk.PhotoImage: return ImageTk.PhotoImage(self._img)
+
+    def ToPhotoImage(self, master=None, **kwargs) -> ImageTk.PhotoImage: return ImageTk.PhotoImage(master=master, image=self._img, **kwargs)
+
+
 
     @classmethod
-    def FromFile(cls, path: Union[str, Path], *, width: int = None, height: int = None) -> 'ImageObject':
+    @overload
+    def FromFile(cls, path: Union[str, Path], *, width: int = None, height: int = None, AsPhotoImage) -> ImageTk.PhotoImage: ...
+    @classmethod
+    @overload
+    def FromFile(cls, path: Union[str, Path], *, width: int = None, height: int = None) -> Image: ...
+
+    @classmethod
+    def FromFile(cls, path: Union[str, Path], *, width: int = None, height: int = None, AsPhotoImage=None):
         Assert(path, str, Path)
 
         with open(path, 'rb') as f:
             with img_open(f) as img:
-                return cls(img, width, height)
+                o = cls(img, width, height)
+                if AsPhotoImage is not None: return o.ToPhotoImage(master=AsPhotoImage, width=width, height=height)
+                return o.Raw
+
+
 
     @classmethod
-    def FromBase64(cls, data: Union[str, bytes], width: int = None, height: int = None) -> 'ImageObject':
+    @overload
+    def FromBase64(cls, path: Union[str, bytes], *, width: int = None, height: int = None, AsPhotoImage) -> ImageTk.PhotoImage: ...
+    @classmethod
+    @overload
+    def FromBase64(cls, path: Union[str, bytes], *, width: int = None, height: int = None) -> Image: ...
+
+    @classmethod
+    def FromBase64(cls, data: Union[str, bytes], width: int = None, height: int = None, AsPhotoImage=None):
         Assert(data, str, bytes)
+        return cls.FromBytes(base64.b64decode(data), width=width, height=height, AsPhotoImage=AsPhotoImage)
+
 
-        return cls.FromBytes(base64.b64decode(data), width=width, height=height)
 
     @classmethod
-    def FromURL(cls, url: str, width: int = None, height: int = None) -> 'ImageObject':
+    @overload
+    def FromURL(cls, path: Union[str, bytes], *, width: int = None, height: int = None, AsPhotoImage) -> ImageTk.PhotoImage: ...
+    @classmethod
+    @overload
+    def FromURL(cls, path: Union[str, bytes], *, width: int = None, height: int = None) -> Image: ...
+
+    @classmethod
+    def FromURL(cls, url: str, width: int = None, height: int = None, AsPhotoImage=None):
         Assert(url, str)
+        return cls.FromBytes(urlopen(url).read(), width=width, height=height, AsPhotoImage=AsPhotoImage)
+
 
-        return cls.FromBytes(urlopen(url).read(), width=width, height=height)
 
     @classmethod
-    def FromBytes(cls, data: bytes, *ext: ImageExtensions, width: int = None, height: int = None) -> 'ImageObject':
-        Assert(data, bytes)
+    @overload
+    def FromBytes(cls, path: Union[str, bytes], *formats: str, width: int = None, height: int = None, AsPhotoImage) -> ImageTk.PhotoImage: ...
+    @classmethod
+    @overload
+    def FromBytes(cls, path: Union[str, bytes], *formats: str, width: int = None, height: int = None) -> Image: ...
 
-        if ext:
+    @classmethod
+    def FromBytes(cls, data: bytes, *formats: str, width: int = None, height: int = None, AsPhotoImage=None):
+        Assert(data, bytes)
+        if formats:
             with BytesIO(data) as buf:
-                with img_open(buf, formats=[e.value for e in ext]) as img:
-                    return cls(img, width, height)
+                with img_open(buf, formats=formats) as img:
+                    o = cls(img, width, height)
+                    if AsPhotoImage is not None: return o.ToPhotoImage(master=AsPhotoImage, width=width, height=height)
+                    return o.Raw
 
         else:
             with BytesIO(data) as buf:
                 with img_open(buf) as img:
-                    return cls(img, width, height)
-
-    @classmethod
-    def Open(cls, path: Union[str, Path]):
-        o = cls.FromFile(path)
-        return o.__call__(path).__enter__()
+                    o = cls(img, width, height)
+                    if AsPhotoImage is not None: return o.ToPhotoImage(master=AsPhotoImage, width=width, height=height)
+                    return o.Raw
 
 
 
@@ -265,7 +229,6 @@ class ImageObject(object):
         init()
         return tuple(EXTENSION.keys())
 
-
     @staticmethod
     @overload
     def IsImage(path: str, *file_types: str) -> bool: ...
